/**
 * Render sass source into file.
 * @memberof apeman-commons-compiling
 * @function renderSass
 * @param {string} src - Source filename or source string.
 * @param {string} dest - Destination file path.
 * @param {object} [options] - Optional settings.
 * @param {boolean} [options.scss=false] - Parse as scsss.
 * @param {string[]} [options.includePaths] - Paths to include.
 * @param {boolean} [options.log] - Should emit log.
 * @param {function} [callback] - Callback when done.
 */

"use strict";

var async = require('async'),
    fs = require('fs'),
    path = require('path'),
    writeout = require('writeout'),
    filedel = require('filedel'),
    execcli = require('execcli'),
    argx = require('argx'),
    colorprint = require('colorprint'),
    hasbin = require('hasbin');

/** @lends renderSass */
function renderSass(src, dest, options, callback) {
    var args = argx(arguments);
    callback = args.pop('function') || argx.noop;
    options = args.pop('object') || {};
    src = args.shift();
    dest = args.pop('string');

    if (!dest) {
        callback(new Error('dest is required.'));
        return;
    }

    var tmpFile = path.resolve([
        '.css.tmp',
        new Date().getTime(),
        'css'
    ].join('.'));
    var isScss = options.scss || false;
    async.waterfall([
        function (callback) {
            fs.exists(src, function (exists) {
                if (exists) {
                    isScss = (path.extname(src) === '.scss') || isScss;
                    fs.readFile(src, callback);
                } else {
                    callback(null, src);
                }
            });
        },
        function (src, callback) {
            console.log('isScss', isScss);
            _sass(src, tmpFile, {
                scss: isScss,
                loadPath: [].concat(options.includePaths || process.cwd()).join(' ')
            }, callback);
        },
        function (content, callback) {
            writeout(dest, String(content), {
                mkdirp: true,
                skipIfIdentical: true
            }, callback);
        },
        function (result, callback) {
            var shouldLog = !result.skipped && options.log;
            if (shouldLog) {
                colorprint.debug('File generated:', dest);
            }
            callback(null, result);
        }
    ], function (err, result) {
        filedel(tmpFile, function () {
            callback(err, result);
        });
    });
}

function _sass(src, dest, options, callback) {
    hasbin('sass', function (hasSass) {
        if (!hasSass) {
            callback(new Error('`sass` not found. Please try `gem install sass`.'))
            return;
        }
        var tmpFile = path.resolve([
            '.sass.tmp',
            new Date().getTime(),
            'tmp'
        ].join('.'));
        async.series([
            function (callback) {
                fs.writeFile(tmpFile, src, callback);
            },
            function (callback) {
                execcli('sass', [tmpFile, dest, '--sourcemap=none', options], callback);
            }
        ], function (err) {
            fs.readFile(dest, function (readErr, content) {
                filedel(tmpFile, function () {
                    callback(err || readErr, content);
                });
            });
        });
    });
}

module.exports = renderSass;
